import { hooks } from "./component";
import { cssSymbol } from "./css";
import { REQUEST_CONSUME } from "./provider";
export function useAttribute(name, converter) {
    return hooks({
        oncreate(h, c, i) {
            const m = new MutationObserver(() => {
                const newValue = converter
                    ? converter(c.getAttribute(name))
                    : c.getAttribute(name);
                if (h.values[i] !== newValue) {
                    h.values[i] = newValue;
                    c.update();
                }
            });
            m.observe(c, { attributes: true, attributeFilter: [name] });
            h.cleanup[i] = () => m.disconnect();
            return converter ? converter(c.getAttribute(name)) : c.getAttribute(name);
        }
    });
}
export const useProperty = (name) => hooks({
    oncreate(h, c, i) {
        const initialValue = c[name];
        Object.defineProperty(c, name, {
            get() {
                return h.values[i];
            },
            set(newValue) {
                if (h.values[i] !== newValue) {
                    h.values[i] = newValue;
                    c.update();
                }
            }
        });
        return initialValue;
    }
});
export const useDispatchEvent = (name, eventInit = {}) => hooks({
    oncreate: (_, c) => (detail) => c.dispatchEvent(new CustomEvent(name, Object.assign({}, eventInit, { detail })))
});
const enabledAdoptedStyleSheets = "adoptedStyleSheets" in Document.prototype &&
    "replace" in CSSStyleSheet.prototype;
export const useStyle = (cssCreator) => hooks({
    oncreate(h, c, i) {
        const css = cssCreator()[cssSymbol];
        if (enabledAdoptedStyleSheets) {
            const styleSheet = new CSSStyleSheet();
            styleSheet.replace(css);
            c.$root.adoptedStyleSheets = [
                ...c.$root.adoptedStyleSheets,
                styleSheet
            ];
        }
        else {
            h.effects[i] = () => {
                const style = document.createElement("style");
                style.textContent = css;
                c.$root.appendChild(style);
            };
        }
        return undefined;
    }
});
const refPrefix = `ref:${String(Math.random()).slice(2)}:`;
export const useRef = (initialValue) => hooks({
    oncreate: (_h, _c, i) => Object.defineProperties(Object.create(null), {
        current: {
            value: initialValue,
            writable: true
        },
        toString: {
            value: () => refPrefix + i
        }
    }),
    onupdate: (h, c, i) => {
        h.effects[i] = () => {
            const value = c.$root.querySelector(`[ref="${refPrefix + i}"]`);
            if (value != null) {
                /* eslint-disable-next-line @typescript-eslint/no-explicit-any */
                h.values[i].current = value;
            }
        };
        return h.values[i];
    }
});
export const useState = (initialState) => hooks({
    oncreate: (h, c, i) => [
        typeof initialState === "function"
            ? initialState()
            : initialState,
        function setState(nextState) {
            const state = h.values[i][0];
            if (typeof nextState === "function") {
                nextState = nextState(state);
            }
            if (state !== nextState) {
                h.values[i][0] = nextState;
                c.update();
            }
        }
    ]
});
export const useReducer = (reducer, initialState) => hooks({
    oncreate: (h, c, i) => [
        initialState,
        function dispatch(action) {
            const state = h.values[i][0];
            const nextState = reducer(state, action);
            if (state !== nextState) {
                h.values[i][0] = nextState;
                c.update();
            }
        }
    ]
});
export const useContext = (context) => hooks({
    oncreate: (h, c, i) => {
        h.cleanup[i] = () => h.deps[i][0].unsubscribe(c);
        c.dispatchEvent(new CustomEvent(REQUEST_CONSUME, {
            bubbles: true,
            composed: true,
            detail: {
                context,
                consumer: c,
                register(provider) {
                    h.deps[i] = [provider];
                }
            }
        }));
        return context.initialValue;
    },
    onupdate: (h, _, i) => h.deps[i][0].value
});
const depsChanged = (prev, next) => prev == null || next.some((f, i) => f !== prev[i]);
export const useEffect = (handler, deps) => hooks({
    onupdate(h, _, i) {
        if (depsChanged(h.deps[i], deps)) {
            h.deps[i] = deps;
            h.effects[i] = handler;
        }
        return undefined;
    }
});
export const useMemo = (fn, deps) => hooks({
    onupdate(h, _, i) {
        let value = h.values[i];
        if (depsChanged(h.deps[i], deps)) {
            h.deps[i] = deps;
            value = fn();
        }
        return value;
    }
});
export const useCallback = (callback, deps) => useMemo(() => callback, deps);
//# sourceMappingURL=hooks.js.map