'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var litHtml = require('lit-html');

let currentCursor;
let currentComponent;
function hooks(config) {
    const h = currentComponent.hooks;
    const index = currentCursor++;
    if (h.values.length <= index && config.oncreate) {
        h.values[index] = config.oncreate(h, currentComponent, index);
    }
    if (config.onupdate) {
        h.values[index] = config.onupdate(h, currentComponent, index);
    }
    return h.values[index];
}
class Component extends HTMLElement {
    constructor() {
        super(...arguments);
        this.updating = false;
        this.$root = this.attachShadow({ mode: "open" });
        this.hooks = {
            values: [],
            deps: [],
            effects: [],
            cleanup: []
        };
    }
    connectedCallback() {
        this.update();
    }
    disconnectedCallback() {
        this.hooks.cleanup.forEach(f => f());
    }
    update() {
        if (this.updating)
            return;
        this.updating = true;
        Promise.resolve().then(() => {
            try {
                currentCursor = 0;
                currentComponent = this;
                this.render();
                this.affect();
            }
            catch (e) {
                console.error(e);
            }
            this.updating = false;
        });
    }
    affect() {
        const h = this.hooks;
        for (let i = 0; i < h.effects.length; i++) {
            if (h.effects[i]) {
                h.cleanup[i] && h.cleanup[i]();
                const cleanup = h.effects[i]();
                if (cleanup) {
                    h.cleanup[i] = cleanup;
                }
                delete h.effects[i];
            }
        }
    }
}

const cssSymbol = Symbol("css");
const hasCSSSymbol = (value) => {
    return typeof value === "object" && value != null && cssSymbol in value;
};
const resolve = (value) => {
    if (typeof value === "number")
        return String(value);
    if (hasCSSSymbol(value))
        return value[cssSymbol];
    if (value instanceof URL)
        return value.href;
    throw new TypeError(`${value} is not supported type.`);
};
const css = (strings, ...values) => ({
    [cssSymbol]: strings
        .slice(1)
        .reduce((acc, s, i) => acc + resolve(values[i]) + s, strings[0])
});
const unsafeCSS = (css) => ({ [cssSymbol]: css });

const REQUEST_CONSUME = "context:request-consume:" + String(Math.random()).slice(2);
class Provider extends HTMLElement {
    constructor() {
        super();
        this.consumers = new Set();
        this.attachShadow({ mode: "open" }).appendChild(document.createElement("slot"));
    }
    connectedCallback() {
        this.addEventListener(REQUEST_CONSUME, e => {
            const { detail: { context, consumer, register } } = e;
            if (this.contextId === context.id) {
                e.stopPropagation();
                this.consumers.add(consumer);
                register(this);
            }
        });
    }
    unsubscribe(consumer) {
        this.consumers.delete(consumer);
    }
    get value() {
        return this._value;
    }
    set value(newValue) {
        if (this._value !== newValue) {
            this._value = newValue;
            this.consumers.forEach(c => c.update());
        }
    }
}

let contextId = 0;
function createContext(initialValue) {
    const id = contextId++;
    return {
        initialValue,
        id,
        defineProvider(name) {
            window.customElements.define(name, class extends Provider {
                get contextId() {
                    return id;
                }
            });
        }
    };
}

function useAttribute(name, converter) {
    return hooks({
        oncreate(h, c, i) {
            const m = new MutationObserver(() => {
                const newValue = converter
                    ? converter(c.getAttribute(name))
                    : c.getAttribute(name);
                if (h.values[i] !== newValue) {
                    h.values[i] = newValue;
                    c.update();
                }
            });
            m.observe(c, { attributes: true, attributeFilter: [name] });
            h.cleanup[i] = () => m.disconnect();
            return converter ? converter(c.getAttribute(name)) : c.getAttribute(name);
        }
    });
}
const useProperty = (name) => hooks({
    oncreate(h, c, i) {
        const initialValue = c[name];
        Object.defineProperty(c, name, {
            get() {
                return h.values[i];
            },
            set(newValue) {
                if (h.values[i] !== newValue) {
                    h.values[i] = newValue;
                    c.update();
                }
            }
        });
        return initialValue;
    }
});
const useDispatchEvent = (name, eventInit = {}) => hooks({
    oncreate: (_, c) => (detail) => c.dispatchEvent(new CustomEvent(name, Object.assign({}, eventInit, { detail })))
});
const enabledAdoptedStyleSheets = "adoptedStyleSheets" in Document.prototype &&
    "replace" in CSSStyleSheet.prototype;
const useStyle = (cssCreator) => hooks({
    oncreate(h, c, i) {
        const css = cssCreator()[cssSymbol];
        if (enabledAdoptedStyleSheets) {
            const styleSheet = new CSSStyleSheet();
            styleSheet.replace(css);
            c.$root.adoptedStyleSheets = [
                ...c.$root.adoptedStyleSheets,
                styleSheet
            ];
        }
        else {
            h.effects[i] = () => {
                const style = document.createElement("style");
                style.textContent = css;
                c.$root.appendChild(style);
            };
        }
        return undefined;
    }
});
const refPrefix = `ref:${String(Math.random()).slice(2)}:`;
const useRef = (initialValue) => hooks({
    oncreate: (_h, _c, i) => Object.defineProperties(Object.create(null), {
        current: {
            value: initialValue,
            writable: true
        },
        toString: {
            value: () => refPrefix + i
        }
    }),
    onupdate: (h, c, i) => {
        h.effects[i] = () => {
            const value = c.$root.querySelector(`[ref="${refPrefix + i}"]`);
            if (value != null) {
                /* eslint-disable-next-line @typescript-eslint/no-explicit-any */
                h.values[i].current = value;
            }
        };
        return h.values[i];
    }
});
const useState = (initialState) => hooks({
    oncreate: (h, c, i) => [
        typeof initialState === "function"
            ? initialState()
            : initialState,
        function setState(nextState) {
            const state = h.values[i][0];
            if (typeof nextState === "function") {
                nextState = nextState(state);
            }
            if (state !== nextState) {
                h.values[i][0] = nextState;
                c.update();
            }
        }
    ]
});
const useReducer = (reducer, initialState) => hooks({
    oncreate: (h, c, i) => [
        initialState,
        function dispatch(action) {
            const state = h.values[i][0];
            const nextState = reducer(state, action);
            if (state !== nextState) {
                h.values[i][0] = nextState;
                c.update();
            }
        }
    ]
});
const useContext = (context) => hooks({
    oncreate: (h, c, i) => {
        h.cleanup[i] = () => h.deps[i][0].unsubscribe(c);
        c.dispatchEvent(new CustomEvent(REQUEST_CONSUME, {
            bubbles: true,
            composed: true,
            detail: {
                context,
                consumer: c,
                register(provider) {
                    h.deps[i] = [provider];
                }
            }
        }));
        return context.initialValue;
    },
    onupdate: (h, _, i) => h.deps[i][0].value
});
const depsChanged = (prev, next) => prev == null || next.some((f, i) => f !== prev[i]);
const useEffect = (handler, deps) => hooks({
    onupdate(h, _, i) {
        if (depsChanged(h.deps[i], deps)) {
            h.deps[i] = deps;
            h.effects[i] = handler;
        }
        return undefined;
    }
});
const useMemo = (fn, deps) => hooks({
    onupdate(h, _, i) {
        let value = h.values[i];
        if (depsChanged(h.deps[i], deps)) {
            h.deps[i] = deps;
            value = fn();
        }
        return value;
    }
});
const useCallback = (callback, deps) => useMemo(() => callback, deps);

function defineElement(name, fn) {
    window.customElements.define(name, class extends Component {
        render() {
            litHtml.render(fn(), this.$root, { eventContext: this });
        }
    });
}

Object.defineProperty(exports, 'html', {
    enumerable: true,
    get: function () {
        return litHtml.html;
    }
});
exports.Component = Component;
exports.createContext = createContext;
exports.css = css;
exports.defineElement = defineElement;
exports.unsafeCSS = unsafeCSS;
exports.useAttribute = useAttribute;
exports.useCallback = useCallback;
exports.useContext = useContext;
exports.useDispatchEvent = useDispatchEvent;
exports.useEffect = useEffect;
exports.useMemo = useMemo;
exports.useProperty = useProperty;
exports.useReducer = useReducer;
exports.useRef = useRef;
exports.useState = useState;
exports.useStyle = useStyle;
